import { Model } from "./Model";
import { FieldAutocomplete } from "./fields/FieldAutocomplete";
import { FieldColorPicker } from "./fields/FieldColorPicker";
import { FieldUploadImage } from "./fields/FieldUploadImage";
import { FieldTextField } from "./fields/FieldTextField";
import { FieldCheckbox } from "./fields/FieldCheckbox";
import { FieldTextArea } from "./fields/FieldTextArea";
import { FieldConsent } from "./fields/FieldConsent";
import { FieldCreator } from "./fields/FieldCreator";
import { FieldListing } from "./fields/FieldListing";
import { FieldEditor } from "./fields/FieldEditor";
import { FieldRadios } from "./fields/FieldRadios";
import { FieldSelect } from "./fields/FieldSelect";
import { FieldToggle } from "./fields/FieldToggle";
export class Field extends Model {
    constructor(title, key, type, config = {}) {
        super(config);
        this._activeFor = {};
        this._model = null;
        this._types = [
            'Autocomplete',
            'ColorPicker',
            'Checkbox',
            'Consent',
            'Creator',
            'Editor',
            'Listing',
            'Radios',
            'Select',
            'TextArea',
            'TextField',
            'Toggle'
        ];
        // Types we know that the backend uses.
        this._typeMaps = [
            'Text',
            'CheckBox',
            'RadioBox',
            'Selection',
            'TeamSelection',
        ];
        // Map those types to the types used by this framework.
        this._typeMap = {
            'Text': 'TextField',
            'CheckBox': 'Checkbox',
            'RadioBox': 'Radios',
            'Selection': 'Select',
            'TeamSelection': 'Select',
        };
        this.formatError = (e) => {
            return !!e?.errors?.[this._key];
        };
        this.formatErrorMessage = (e, show) => {
            return show ? e?.errors?.[this._key] : null;
        };
        this._class = 'Field';
        this._title = title;
        this._type = type;
        this._key = key;
        this.handleTypes();
        this.resetToDefault();
        this._config.class = this._config.class ?? 'pt-4';
        this._config.outlined = this._config.outlined ?? true;
        this._config.singleLine = this._config.singleLine ?? false;
        this._config.formatError = this._config.formatError ?? this.formatError;
        this._config.formatErrorMessage = this._config.formatErrorMessage ?? this.formatErrorMessage;
    }
    handleTypes() {
        if (this._typeMaps.includes(this._type)) {
            this._type = this._typeMap[this._type];
        }
        if (this._types.includes(this._type)) {
            this._typeModel = this.setTypeModel();
        }
        else if (this._type !== "") {
            console.error(`[BREAKING] Unknown field type: ${this._type}`);
        }
    }
    resetToDefault() {
        this._model = this._config.default ?? null;
        if (this._model === null) {
            this._model = this._config.defaultFunction ? this._config.default() : this._config.default ?? null;
            if (this._typeModel && this._model)
                this._model = this._typeModel.default(this._model, this._config.bind);
        }
        else {
            this._model = this._config.defaultFunction ? this._config.default(this._model) : this._model;
            if (this._typeModel && this._model)
                this._model = this._typeModel.default(this._model, this._config.bind);
        }
    }
    setTypeModel() {
        switch (this._type) {
            case 'Autocomplete':
                return new FieldAutocomplete(this);
            case 'ColorPicker':
                return new FieldColorPicker(this);
            case 'UploadImage':
                return new FieldUploadImage(this);
            case 'TextField':
                return new FieldTextField(this);
            case 'TextArea':
                return new FieldTextArea(this);
            case 'Checkbox':
                return new FieldCheckbox(this);
            case 'Listing':
                return new FieldListing(this);
            case 'Consent':
                return new FieldConsent(this);
            case 'Creator':
                return new FieldCreator(this);
            case 'Editor':
                return new FieldEditor(this);
            case 'Radios':
                return new FieldRadios(this);
            case 'Select':
                return new FieldSelect(this);
            case 'Toggle':
                return new FieldToggle(this);
            default:
                return null;
        }
    }
}
//# sourceMappingURL=Field.js.map