export class Requestor {
    constructor(doIf, storeGet, storeSet = () => false, axiosGet = () => false, axiosPost = () => false, onThen = () => false, onCatch = () => false, fallback = () => null, overwrite = () => false) {
        this.isDone = false;
        this.isError = false;
        this._doIf = doIf;
        this._storeGet = storeGet;
        this._storeSet = storeSet() ? storeSet : storeGet();
        this._axiosGet = axiosGet;
        this._axiosPost = axiosPost;
        this._onThen = onThen;
        this._onCatch = onCatch;
        this._fallback = fallback;
        this._overwrite = overwrite;
    }
    static inside(data, has) {
        if (typeof has === 'string')
            has = has.split('.');
        let temp = data;
        let fail = false;
        has.forEach((r) => {
            if (!temp)
                fail = true;
            if (!fail && r in temp) {
                temp = temp[r];
            }
            else {
                temp = null;
            }
        });
        return temp;
    }
    handle($this, onComplete, ignoreFailure = false, ignoreStore = false) {
        if (!this._doIf()) {
            onComplete();
            return;
        }
        let data = undefined;
        let done = (onComplete, error = false) => {
            this.isDone = !error;
            this.isError = error;
            onComplete();
        };
        if (this._storeGet() !== false && !ignoreStore) {
            if (typeof this._storeGet() === 'string') {
                data = Requestor.inside($this, `c.request.store.${this._storeGet()}`);
            }
            else {
                this._storeGet().forEach((t) => {
                    if (!data)
                        data = Requestor.inside($this, t);
                });
            }
        }
        if (!data || this._overwrite()) {
            if (this._axiosGet()) {
                $this.$axios
                    .get(this._axiosGet())
                    .then((response) => {
                    this._onThen(response.data);
                    if (this._storeSet())
                        $this.c.request.store[this._storeSet()] = response.data;
                    done(onComplete);
                })
                    .catch((response) => {
                    if (ignoreFailure) {
                        done(onComplete);
                    }
                    else if (Requestor.inside(response, 'response.data') && !$this.c.middleware.handleControlledErrors(response.response.data, onComplete)) {
                        this._onCatch(response);
                        done(onComplete, true);
                    }
                });
            }
            else if (this._axiosPost()) {
                $this.$axios
                    .post(this._axiosPost().url, this._axiosPost().params)
                    .then((response) => {
                    this._onThen(response.data);
                    if (this._storeSet())
                        $this.c.request.store[this._storeSet()] = response.data;
                    done(onComplete);
                })
                    .catch((response) => {
                    if (ignoreFailure) {
                        done(onComplete);
                    }
                    else if (!$this.c.middleware.handleControlledErrors(response.response.data, onComplete)) {
                        this._onCatch(response);
                        done(onComplete, true);
                    }
                });
            }
            else if (this._fallback()) {
                $this.c.request.store[this._storeSet()] = this._fallback();
                done(onComplete);
            }
            else {
                done(onComplete, true);
            }
        }
        else {
            done(onComplete);
        }
    }
}
//# sourceMappingURL=Requestor.js.map