export class Requestor
{
    public isDone: boolean = false;
    public isError: boolean = false;

    public _doIf: any;
    public _storeGet: any;
    public _storeSet: any;
    public _axiosGet: any;
    public _axiosPost: any;
    public _onThen: any;
    public _onCatch: any;
    public _fallback: any;
    public _overwrite: any;

    constructor(
        doIf: any,
        storeGet: any,
        storeSet: any = () => false,
        axiosGet: any = () => false,
        axiosPost: any = () => false,
        onThen: any = () => false,
        onCatch: any = () => false,
        fallback: any = () => null,
        overwrite: any = () => false
    ) {
        this._doIf = doIf;
        this._storeGet = storeGet;
        this._storeSet = storeSet() ? storeSet : storeGet();
        this._axiosGet = axiosGet;
        this._axiosPost = axiosPost;
        this._onThen = onThen;
        this._onCatch = onCatch;
        this._fallback = fallback;
        this._overwrite = overwrite;
    }

    static inside(data: any, has: any) {
        if (typeof has === 'string') has = has.split('.');
        let temp = data;
        let fail = false;
        has.forEach((r: any) => {
            if (!temp) fail = true;
            if (!fail && r in temp) {
                temp = temp[r];
            } else {
                temp = null;
            }
        });
        return temp;
    }

    handle($this: any, onComplete: any, ignoreFailure: boolean = false, ignoreStore: boolean = false)
    {
        if (!this._doIf()) {
            onComplete();
            return;
        }

        let data: any = undefined;
        let done = (onComplete: any, error: boolean = false) => {
            this.isDone = !error;
            this.isError = error;
            onComplete();
        };

        if (this._storeGet() !== false && !ignoreStore) {
            if (typeof this._storeGet() === 'string') {
                data = Requestor.inside($this, `c.request.store.${this._storeGet()}`)
            } else {
                this._storeGet().forEach((t: any) => {
                    if (!data) data = Requestor.inside($this, t);
                });
            }
        }

        if (!data || this._overwrite()) {
            if (this._axiosGet()) {
                $this.$axios
                    .get(this._axiosGet())
                    .then((response: any) => {
                        this._onThen(response.data);
                        if (this._storeSet()) $this.c.request.store[this._storeSet()] = response.data;
                        done(onComplete);
                    })
                    .catch((response: any) => {
                        if (ignoreFailure) {
                            done(onComplete);
                        } else if (Requestor.inside(response, 'response.data') && !$this.c.middleware.handleControlledErrors(response.response.data, onComplete)) {
                            this._onCatch(response);
                            done(onComplete, true);
                        }
                    });
            } else if (this._axiosPost()) {
                $this.$axios
                    .post(this._axiosPost().url, this._axiosPost().params)
                    .then((response: any) => {
                        this._onThen(response.data);
                        if (this._storeSet()) $this.c.request.store[this._storeSet()] = response.data;
                        done(onComplete);
                    })
                    .catch((response: any) => {
                        if (ignoreFailure) {
                            done(onComplete);
                        } else if (!$this.c.middleware.handleControlledErrors(response.response.data, onComplete)) {
                            this._onCatch(response);
                            done(onComplete, true);
                        }
                    });
            } else if (this._fallback()) {
                $this.c.request.store[this._storeSet()] = this._fallback();
                done(onComplete);
            } else {
                done(onComplete, true);
            }
        } else {
            done(onComplete);
        }
    }
}