import {Field}      from "./../../models/Field";
import {Verify}     from "./../../models/Verify";
import {Title}      from "./../../models/elements/Title";
import {FieldBase}  from "./extend/FieldBase";

export class FieldListing extends FieldBase
{
    constructor(field: any = null)
    {
        super(field);
        if (this._field !== null) this.verify();
    }

    // Ensure that the config object is correct.
    public verify()
    {
        this._verify = new Verify(this._field);
        this._verify.keys(['input']);
        this._verify.items();
        this._verify.selector();
    }

    public default(data: any, bind: string = 'undefined')
    {
        let model: any[] = [this._field._config.inputFunction ? this._field._config.input() : this._field._config.input];

        data.forEach((r: any, index: number) => {
            let item: any = {};
            Object.keys(r).forEach((k) => {
                item[k] = {_config: {bind: bind}, _model: r[k]}
            });
            model.push(this.add(item, index + 1));
        })

        return model;
    }

    public format(data: any, $this: any)
    {
        if (!data) return [];
        let objects: any = [];
        data.forEach((r: any) => {
            if (r.uniqueIdentifier !== 'input') {
                let object: any = {};
                Object.keys(r).forEach((k: string) => {
                    if (r[k]._typeModel?.format) {
                        object[k] = r[k]._model;
                    }
                })
                objects.push(object);
            }
        })
        return objects;
    }

    public add(item: any, defaults: number = 0)
    {
        // Extract model value for each key in the headers.
        // Since we pushed the input row to the field model, that is what we get as an item.
        let toBePushed: any = {
            // Ensure that the key is unique - also if we delete elements.
            uniqueIdentifier: defaults
                ? defaults
                : this._field._model.length === 1
                    ? 0
                    : this._field._model[this._field._model.length - 1].uniqueIdentifier + 1
        };
        Object.keys(item).forEach((k: string) => {
            // The key index is not a field model.
            if (k !== 'uniqueIdentifier') {
                let header = this._field._model?.[0]?.[k];
                toBePushed[k] = new Field('', `${k}-${toBePushed.uniqueIdentifier}`, header?._type ?? 'Text', {
                    bind: `${item[k]._config.bind}-listing`,
                    items: () => header?._config?.items ? header?._config?.items() : [],
                    default: item[k]._model,
                    singleLine: true,
                    hidePadding: true,
                    required: false,
                    disabled: true,
                    dense: true,
                    class: header?._config?.class ?? 'mt-1 mb-2 ml-2',
                });
                item[k]._model = null;
            }
        })

        // Put it in readonly fields.
        if (defaults) {
            return toBePushed;
        } else {
            this._field._model.push(toBePushed);
        }
    }

    public remove(item: any)
    {
        this._field._model = this._field._model.filter((r: any) => r.uniqueIdentifier !== item.uniqueIdentifier);
    }

    public contains(uniqueIdentifier: string)
    {
        let find = this._field._model.map((r: any) => r.uniqueIdentifier).indexOf(uniqueIdentifier);
        return find > -1;
    }

    // Check if row has values.
    public required($this: any, item: any)
    {
        return Object.keys(item).every((k: string) => {
            if (item[k]._typeModel?.format) {
                return !!item[k]._model;
            } else {
                return true;
            }
        })
    }

    // Input fields needed to build the config.
    public editorSection($this: any, bind: string)
    {
        super.editorSection($this, bind);

        this._editor.push(new Title('Kolonner', {class: 'pt-2', classInner: 'pl-3'}));
        this._editor.push(new Field('Header', 'listing', 'Listing', {
            bind: `${bind}-config`,
            class: 'pt-4',
            required: false,
            default: $this.item._config.default?.items ?? [],
            input: {
                uniqueIdentifier: 'input',
                value: new Field('Navn', 'value', 'Text', {
                    bind: `${bind}-config`,
                    class: 'mt-1 mb-2 mr-2',
                    dense: true,
                    hideTitle: true,
                    singleLine: true,
                    noErrorMessage: true,
                }),
                order: new Field('Sortering', 'order', 'Text', {
                    bind: `${bind}-config`,
                    class: 'mt-1 mb-2 mr-2',
                    dense: true,
                    hideTitle: true,
                    singleLine: true,
                    noErrorMessage: true,
                }),
            },
            header: [
                {text: '', value: '', type: 'Actions', show: true, width: 50, align: 'center'},
                {text: 'Navn', value: 'value', type: 'Selector', show: true},
                {text: 'Sortering', value: 'order', type: 'Selector', show: true},
            ]
        }));

        return this._editor;
    }
}